package com.mygdx.game;

import com.badlogic.gdx.ApplicationAdapter;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import java.util.ConcurrentModificationException;
import logic.Element;
import logic.World;

public class Game extends ApplicationAdapter {

    private static Game game;
    ShapeRenderer shaper;
    private World world;
    private float cameraSpeed = 15;
    private BitmapFont font;
    private Thread worldThread;
    private boolean paused = false;

    @Override
    public void create() {
        game = this;
        world = new World(2500, 2500);
        shaper = new ShapeRenderer();
        shaper.setAutoShapeType(true);
        font = new BitmapFont();
        worldThread = new Thread(world);
        worldThread.setPriority(Thread.MAX_PRIORITY);
        world.newGen(true);
        worldThread.start();
    }

    @Override
    public void render() {
        // Controls
        if (Gdx.input.isKeyJustPressed(Input.Keys.SPACE)) {
            world.newGen(false);
        }
        if (Gdx.input.isKeyPressed(Input.Keys.RIGHT)) {
            shaper.translate(-cameraSpeed, 0, 0);
        }
        if (Gdx.input.isKeyPressed(Input.Keys.LEFT)) {
            shaper.translate(cameraSpeed, 0, 0);
        }
        if (Gdx.input.isKeyPressed(Input.Keys.UP)) {
            shaper.translate(0, -cameraSpeed, 0);
        }
        if (Gdx.input.isKeyPressed(Input.Keys.DOWN)) {
            shaper.translate(0, cameraSpeed, 0);
        }
        if (Gdx.input.isKeyJustPressed(Input.Keys.PLUS)) {
            shaper.scale(0.5f, 0.5f, 1);
        }
        if (Gdx.input.isKeyJustPressed(Input.Keys.MINUS)) {
            shaper.scale(1.5f, 1.5f, 1);
        }
        if (Gdx.input.isKeyJustPressed(Input.Keys.P)) {
            paused = !paused;
        }
        if (Gdx.input.isKeyJustPressed(Input.Keys.L)) {
            if (world.getFpsLimit() > 0) {
                Log.log(Log.INFO, "Disabling FPS Limit");
                world.setFpsLimit(0);
            } else {
                Log.log(Log.INFO, "Setting FPS Limit to 60");
                world.setFpsLimit(60);
            }
        }
        // Draw
        Gdx.gl.glClearColor(0, 0, 0, 1);
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
        shaper.setColor(1, 1, 1, 1);
        shaper.begin(ShapeRenderer.ShapeType.Line);
        try {
            for (Element e : world.getElements()) {
                try {
                    e.render(shaper);
                } catch (ArrayIndexOutOfBoundsException ex) {
                    // Killed while rendering
                    //Log.log(Log.ERROR, ex+"");
                }
            }
        } catch (ConcurrentModificationException e) {
        }
        shaper.setColor(0.3f, 0.3f, 0.3f, 1);
        // draw borders
        shaper.rect(0, 0, world.getWidth(), world.getHeight());
        shaper.end();
    }

    public World getWorld() {
        return world;
    }

    public static Game get() {
        return game;
    }

    public boolean isPaused() {
        return paused;
    }
}
